.\"-
.\" Copyright (c) 2012 Konstantin Belousov <kib@FreeBSD.org>
.\"
.\" Redistribution and use in source and binary forms, with or without
.\" modification, are permitted provided that the following conditions
.\" are met:
.\" 1. Redistributions of source code must retain the above copyright
.\"    notice, this list of conditions and the following disclaimer.
.\" 2. Redistributions in binary form must reproduce the above copyright
.\"    notice, this list of conditions and the following disclaimer in the
.\"    documentation and/or other materials provided with the distribution.
.\"
.\" THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
.\" IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
.\" OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
.\" IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
.\" INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
.\" NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
.\" DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
.\" THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
.\" (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
.\" THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
.\"
.Dd June 17, 2014
.Dt PSTACK 1
.Os
.Sh NAME
.Nm pstack
.Nd print the backtrace of the running program
.Sh SYNOPSIS
.Nm
.Op Fl loOtv
.Op Fl a Ar "arg count"
.Op Fl f Ar "frame count"
.Ar pid
.Sh DESCRIPTION
The
.Nm
utility prints the stack backtraces for the running program.
The
.Nm
attaches to the traced process using the
.Fn ptrace 2
debugging interface.
If possible, the symbolic name and offset for the function executing in
each frame is displayed.
The
.Nm
can be directed to print the shared object name, which contains
the function.
.Pp
The first line of the output displays the
.Va pid
of the traced process, binary name as reported by the kernel,
and the currently active
.Va osrel
version of the ABI for the process.
After that, for each thread in the process, the thread id is displayed,
followed by the backtrace.
.Pp
The following options are available:
.Bl -tag -width indent
.It Fl a Ar count
Try to guess the arguments of each function appearing in the backtrace.
Not more than
.Ar count
arguments are printed for each function.
Ability to correctly read the argument values depends on the
architecture and compiler optimization, but it is generally impossible
to do.
.It Fl f Ar count
For each thread, print not more than
.Ar count
frames in the backtrace.
.It Fl l
Instead of walking the stacks of the running threads, print the names
of the shared objects loaded into the process address space.
The output of the utility resembles the output of
.Xr ldd 1
command run on the binary.
.It Fl o
Show the names of the objects which contain the location of the frame
current instruction pointer.
.It Fl O
Same as
.Fl o ,
but displays the full path to the objects instead of only names.
.It Fl t
Measure and print the time which the target process was suspended
to walk the stack frames.
.It Fl v
Be verbose.
.El
.Sh SEE ALSO
.Xr gdb 1 ,
.Xr ldd 1 ,
.Xr ptrace 2 ,
.Xr libunwind 3 ,
.Xr procfs 5
.Sh AUTHORS
The
.Nm
command is reimplemented by
.An Konstantin Belousov
to mimic the user interface of other similar
.Nm
utilities, in particular, the
.Nm
written by
.An Peter Edwards .
This version relies on the
.Nm libunwind
library as the stack unwinding backend.
.Sh CAVEATS
On i386, the argument printing code assumes that the frame layout is
standard, which is currently often not the case due to the padding
added between return address and previous frame pointer, as done by
compilers to align the stack.
.Pp
On amd64, the argument printing currently does not work.
The amd64 ABI specifies that integer arguments are passed in the registers
which are not required to be preserved across function calls.
As the result, there is typically no way to re-create the arguments values
on any frame except the current one.
.Pp
The symbol names cannot be inferred if the corresponding binary or
shared object are already deleted, since
.Nm libunwind
needs to read the symbols from the on-disk ELF image.
This could typically occur if the binary or library is reinstalled,
but the process uses older version of the binary with the same name.
.Pp
The
.Nm
only works on the processes executing ELF binaries.
.Pp
You cannot get a backtrace for the process, to which another debugger
is currently attached.
The
.Xr ptrace 2
debugging mechanism used to obtain the neccessary live process information
causes some syscalls to return
.Va EINTR
errors on
.Nm
detach.
.Pp
The
.Nm
utility cannot yet extract a backtrace from the core dump.
.Pp
The architecture of the program and
.Nm
shall match (this is
.Nm libunwind
restriction).
